﻿using System;
using System.Linq;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.Serialization;
using System.ComponentModel;

namespace TreeMap
{
    [System.Runtime.Serialization.DataContractAttribute(Name = "Tag", Namespace = "http://schemas.datacontract.org/2004/07/Model")]   
    public partial class Tag :  INotifyPropertyChanged
    {
        /// <summary>
        /// The id of the tag
        /// </summary>
        [DataMember]
        public virtual Guid Id { get; set; }

        /// <summary>
        /// The parent Id of the tag <c>Null</c> if root node
        /// </summary>
        [DataMember]
        public virtual Guid? ParentId { get; set; }
        
        public string _name;
        /// <summary>
        /// The tags display name
        /// </summary>
        [DataMember]
        public virtual string Name
        {
            get { return _name; }
            set
            {
                _name = value;
                this.NotifyPropertyChanged("Name");
            }
        }

        private Tag _parent;
        /// <summary>
        /// The parent of this tag
        /// </summary>  
        public Tag Parent
        {
            get { return _parent; }
            set 
            {
                _parent = value;
                NotifyPropertyChanged("Parent"); 
            }
        }

        private ObservableCollection<Tag> _childern;
        /// <summary>
        /// The children of this tag
        /// </summary>  
        public ObservableCollection<Tag> Children
        {
            get
            {
                // IT IS OK TO ASSUME THAT A CHILDREN COLLECTION IS EMPTY AND IT IS NEVER NULL
                if (_childern == null)
                {
                    _childern = new ObservableCollection<Tag>();
                }
                return _childern; 
            }
            private set
            {
                _childern = value;
                NotifyPropertyChanged("Children");
            }
        }

        private int _metric;
        /// <summary>
        /// A metric for tag map
        /// </summary>      
        [DataMember]
        public int Metric
        {
            get { return _metric; }
            set 
            {
                _metric = value;
                NotifyPropertyChanged("Metric"); 
            }
        }

        private bool _selected;
        /// <summary>
        /// Whether the node is selected or not
        /// </summary>
        public bool Selected
        {
            get { return _selected; }
            set 
            {
                _selected = value;
                NotifyPropertyChanged("Selected"); 
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
