﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Linq;
using System.ComponentModel;
using System.Text;
using System.IO;

namespace TreeMap
{
    public class ViewModel: INotifyPropertyChanged
    {
        #region ctor
        public ViewModel()
        {
            CreateTags();
        }
        #endregion

        #region Properties
        private ObservableCollection<Tag> _tags;
        public ObservableCollection<Tag> Tags 
        {
            get { return _tags; }
            private set
            {
                _tags = value;
                OnPropertyChanged("Tags");
            }
        }

        private Tag _rootTag;
        public Tag RootTag
        {
            get { return _rootTag; }
            private set
            {
                _rootTag = value;
                OnPropertyChanged("RootTag");
            }
        }       
        #endregion

        #region Methods
        private Tag _defaultTag = null;

        /// <summary>
        /// Creates a heirarchy of tags
        /// </summary>
        private void CreateTags()
        {            
            List<Tag> allTags = new List<Tag>();

            //Initializing from a string set earlier
            string tagStr = "<ArrayOfTag xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns=\"http://schemas.datacontract.org/2004/07/Model\"><Tag><Id>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</Id><Metric>2243603</Metric><Name>A</Name><ParentId i:nil=\"true\" /></Tag><Tag><Id>2883874a-1834-4a9d-91a3-674b9aa8c396</Id><Metric>500</Metric><Name>B</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>65dd4b1a-d49e-4ecd-ab51-2bde5c85cf85</Id><Metric>239917</Metric><Name>C</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>edaf18f6-73c8-49fd-9840-08c3bc99b492</Id><Metric>315703</Metric><Name>D</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>cb47a395-9be9-4780-acf9-64b2c389174c</Id><Metric>24880</Metric><Name>E</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>0e944281-afd1-4edc-a2b8-e826aceee28c</Id><Metric>4798</Metric><Name>F</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>8ea0a517-299e-4cfb-9e00-9f37956acb3b</Id><Metric>9899</Metric><Name>G</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>84ed62d3-6489-47c3-bdf0-ed85a2ef6189</Id><Metric>1333748</Metric><Name>H</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>b87bce74-09e1-4739-bba4-02d830b65d19</Id><Metric>49950</Metric><Name>I</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>7d2bf07c-d83d-45ad-96b6-4b5b546184a5</Id><Metric>262614</Metric><Name>J</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>165581a3-5250-4744-b0ff-5c63eb6ca61c</Id><Metric>1601</Metric><Name>K</Name><ParentId>75e47e17-8d2d-46f5-b5b3-c1556b3bf75a</ParentId></Tag><Tag><Id>20f398a6-66ae-463f-ab40-dc4a65890a5f</Id><Metric>0</Metric><Name>L</Name><ParentId>2883874a-1834-4a9d-91a3-674b9aa8c396</ParentId></Tag><Tag><Id>98084275-1df0-4fc2-9f9a-7ed2d0a5254f</Id><Metric>48131</Metric><Name>M</Name><ParentId>b87bce74-09e1-4739-bba4-02d830b65d19</ParentId></Tag><Tag><Id>a6dd9db3-7a5f-4e1a-8f82-24c6fc0870c0</Id><Metric>30640</Metric><Name>N</Name><ParentId>98084275-1df0-4fc2-9f9a-7ed2d0a5254f</ParentId></Tag></ArrayOfTag>";
            
            using (System.IO.MemoryStream stream = new System.IO.MemoryStream())
            {
                StreamWriter writer = new StreamWriter(stream);
                writer.Write(tagStr);
                writer.Flush();
                stream.Seek(0, SeekOrigin.Begin);
                System.Runtime.Serialization.DataContractSerializer ser =
                    new System.Runtime.Serialization.DataContractSerializer(typeof(List<Tag>));
                allTags = (List<Tag>)ser.ReadObject(stream);
            }       

            IEnumerable<Tag> topLevelTags = allTags.Where(i => i.ParentId == null).OrderBy(i => i.Name);

            BuildHierarchy(topLevelTags, allTags);

            Tags = new ObservableCollection<Tag>(topLevelTags);
            RootTag = topLevelTags.ElementAt(0);
            _defaultTag = RootTag;
        }

        private void BuildHierarchy(IEnumerable<Tag> tags, IEnumerable<Tag> allTags)
        {
            if (tags == null || tags.Count() == 0)
                return;

            foreach (Tag tag in tags)
            {
                foreach (Tag toAdd in allTags.Where(i => i.ParentId == tag.Id).OrderBy(i => i.Name))
                {
                    tag.Children.Add(toAdd);
                }

                // set parent
                foreach (Tag childTag in tag.Children)
                    childTag.Parent = tag;

                BuildHierarchy(tag.Children, allTags);
            }
        }
        #endregion

        /******************************************************************************************************/
        /// <summary>
        /// The drill down event that is causing problems
        /// </summary>
        /// <param name="tag"></param>
        public void DrillDown(Tag tag)
        {
            if (tag == null)
                return;
            //If the following line is commented out then there is no issue
            tag.Selected = false;

            RootTag = tag;
        }
        /******************************************************************************************************/

        public void ResetDrill()
        {
            RootTag = _defaultTag;
        }

        #region Property Changed events
        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propName));
        }
        #endregion
    }
}
